import json

from django.test import TestCase
from django.test.client import RequestFactory
from freezegun import freeze_time
from mock import Mock, patch

from tsuru_dashboard.auth.views import LoginRequiredView
from tsuru_dashboard.dashboard.views import DeploysView


class DeploysViewTest(TestCase):

    def setUp(self):
        self.factory = RequestFactory()

    def test_requires_login(self):
        assert issubclass(DeploysView, LoginRequiredView), \
            "DeploysView should inherit from LoginRequiredView"

    @patch("requests.get")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_get_no_deploys(self, token_is_valid, get):
        token_is_valid.return_value = True
        resp = Mock()
        resp.json.return_value = None
        get.return_value = resp
        request = self.factory.get("/dashboard/deploys")
        request.session = {"tsuru_token": "sometoken"}
        response = DeploysView.as_view()(request)
        self.assertEqual(200, response.status_code)
        result = json.loads(response.content)
        self.assertEqual({"last_deploys": 0, "errored": 0}, result)

    @freeze_time("2012-04-01 16:32:15", tz_offset=0)
    @patch("requests.get")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_get_filtering(self, token_is_valid, get):
        token_is_valid.return_value = True
        resp = Mock()
        resp.json.return_value = [
            {"Timestamp": "2012-03-31T12:10:15Z",
             "Error": "something went wrong"},
            {"Timestamp": "2012-03-31T09:02:15-0300",
             "Error": None},
            {"Timestamp": "2012-04-01T19:02:15+0800",
             "Error": None},
            {"Timestamp": "2012-03-31T00:02:15-0800",
             "Error": None},
            {"Timestamp": "2012-04-01T10:02:15.903-0300",
             "Error": "a failure in the deploy"},
            {"Timestamp": "2012-03-31T13:32:16-0300",
             "Error": None},
            {"Timestamp": "2012-03-31T20:00:00.903Z",
             "Error": None},
            {"Timestamp": "2012-03-31T20:00:00",
             "Error": "deployment failed"},
        ]
        get.return_value = resp
        request = self.factory.get("/dashboard/deploys")
        request.session = {"tsuru_token": "sometoken"}
        response = DeploysView.as_view()(request)
        self.assertEqual(200, response.status_code)
        result = json.loads(response.content)
        self.assertEqual({"last_deploys": 5, "errored": 2}, result)



import requests

from datetime import datetime, timedelta
from time import mktime


class Prometheus(object):
    def __init__(self, url, query, date_range=None):
        self.url = url
        self.query = query
        self.date_range = date_range

    @property
    def delta(self):
        if not self.date_range:
            return {"hours": 1}

        if "h" in self.date_range:
            return {"hours": int(self.date_range.replace("h", ""))}

        if "d" in self.date_range:
            return {"days": int(self.date_range.replace("d", ""))}

        if "w" in self.date_range:
            return {"days": int(self.date_range.replace("w", ""))*7}

    @property
    def start(self):
        return self.end - timedelta(**self.delta)

    @property
    def end(self):
        return datetime.now()

    @property
    def resolution(self):
        return (self.end - self.start).total_seconds() / 250

    def default_processor(self, results):
        def toMs(x):
            if len(x) == 2:
                return [x[0]*1000, x[1]]
            return x

        return list(map(toMs, results))

    def get_metrics(self, query, processor=None):
        url = self.url
        url += "/api/v1/query_range?"
        url += query
        url += "start={}".format(mktime(self.start.timetuple()))
        url += "&end={}".format(mktime(self.end.timetuple()))
        url += "&step={}".format(self.resolution)
        result = requests.get(url)

        if processor is None:
            result = result.json()['data']['result'][0]['values']
            processor = self.default_processor

        return processor(result)

    def mem_max(self, interval=None):
        data = {"min": 0, "max": 1024}
        query = "query=avg(container_memory_usage_bytes{%s})/1024/1024&" % self.query
        data["avg"] = self.get_metrics(query)

        query = "query=max(container_memory_usage_bytes{%s})/1024/1024&" % self.query
        data["max"] = self.get_metrics(query)

        query = "query=min(container_memory_usage_bytes{%s})/1024/1024&" % self.query
        data["min"] = self.get_metrics(query)
        return {"data": data}

    def cpu_max(self, interval=None):
        data = {"min": 0, "max": 100}
        query = "query=avg(rate(container_cpu_usage_seconds_total{%s}[2m]) * 100)&" % self.query
        data["avg"] = self.get_metrics(query)

        query = "query=max(rate(container_cpu_usage_seconds_total{%s}[2m]) * 100)&" % self.query
        data["max"] = self.get_metrics(query)

        query = "query=min(rate(container_cpu_usage_seconds_total{%s}[2m]) * 100)&" % self.query
        data["min"] = self.get_metrics(query)
        return {"data": data}

    def units(self, interval=None):
        data = {"min": 0, "max": 100}
        query = "query=max(count(rate(container_memory_usage_bytes{%s}[2m])) by (slave))&" % self.query
        data["units"] = self.get_metrics(query)
        return {"data": data}

    def swap(self, interval=None):
        data = {"min": 0, "max": 1024}
        query = "query=avg(container_memory_swap{%s})/1024/1024&" % self.query
        data["avg"] = self.get_metrics(query)

        query = "query=max(container_memory_swap{%s})/1024/1024&" % self.query
        data["max"] = self.get_metrics(query)

        query = "query=min(container_memory_swap{%s})/1024/1024&" % self.query
        data["min"] = self.get_metrics(query)
        return {"data": data}

    def netrx(self, interval=None):
        data = {"min": 0, "max": 100}
        query = "query=rate(container_network_receive_bytes_total{%s}[2m])/1024&" % self.query
        data["netrx"] = self.get_metrics(query)
        return {"data": data}

    def nettx(self, interval=None):
        data = {"min": 0, "max": 100}
        query = "query=rate(container_network_transmit_bytes_total{%s}[2m])/1024&" % self.query
        data["nettx"] = self.get_metrics(query)
        return {"data": data}

    def connections(self, interval=None):
        data = {"min": 0, "max": 100}
        query = 'query=sum(container_connections{%s,state="ESTABLISHED",protocol="tcp"}) by (destination)&' % self.query
        data['data'] = self.get_metrics(query, processor=self.connections_processor)
        return data

    def connections_processor(self, result):
        results = result.json()['data']['result']
        data = {}
        for r in results:
            data[r['metric']['destination']] = self.default_processor(r['values'])
        return data


class AppBackend(Prometheus):
    def __init__(self, app, url, process_name=None, date_range=None):
        query = 'container_label_tsuru_app_name="%s"' % app["name"]
        if process_name is not None:
            query += ',container_label_tsuru_process_name="%s"' % process_name
        return super(AppBackend, self).__init__(
            url=url,
            query=query,
            date_range=date_range,
        )



from django.conf.urls import url
from django.views.decorators.csrf import csrf_exempt

import views


urlpatterns = [
    url(r'^$', views.ListEvent.as_view(), name='event-list'),
    url(r'^kinds/$', views.KindList.as_view(), name='kind-list'),
    url(r'^(?P<uuid>[\w-]+)/$', views.EventInfo.as_view(), name='event-info'),
    url(r'^(?P<uuid>[\w-]+)/cancel/$', csrf_exempt(views.EventCancel.as_view()), name='event-cancel'),
]



from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

from tsuru_dashboard import settings
from tsuru_dashboard.auth.views import ChangePassword
from tsuru_dashboard.auth.forms import ChangePasswordForm

from mock import patch, Mock


class TestResetPasswordView(TestCase):
    @patch("requests.get")
    def test_get(self, get):
        get.return_value = Mock(status_code=200)
        request = RequestFactory().get("/")
        request.session = {"tsuru_token": "ble"}
        response = ChangePassword.as_view()(request)
        self.assertIn('auth/change_password.html',
                      response.template_name)
        self.assertIsInstance(response.context_data['form'],
                              ChangePasswordForm)

    @patch("django.contrib.messages.error")
    @patch("requests.put")
    @patch("requests.get")
    def test_post(self, get, put, error):
        get.return_value = Mock(status_code=200)
        data = {
            "old": "old",
            "new": "new",
            "confirm": "new",
        }
        request = RequestFactory().post("/", data)
        request.session = {'tsuru_token': 'tokentest'}
        response = ChangePassword.as_view()(request)
        headers = {'authorization': 'tokentest'}
        url = "{0}/users/password".format(settings.TSURU_HOST)
        put.assert_called_with(url, data=data, headers=headers)
        self.assertEqual(302, response.status_code)
        self.assertEqual('/auth/change-password/', response.items()[1][1])

    @patch("django.contrib.messages.success")
    @patch("requests.put")
    @patch("requests.get")
    def test_post_sends_success_message(self, get, put, success):
        get.return_value = Mock(status_code=200)
        put.return_value = Mock(status_code=200)
        data = {
            "old": "old",
            "new": "new",
            "confirm": "new",
        }
        request = RequestFactory().post("/", data)
        request.session = {'tsuru_token': 'tokentest'}
        ChangePassword.as_view()(request)
        success.assert_called_with(request, u'Password successfully updated!', fail_silently=True)

    @patch("django.contrib.messages.error")
    @patch("requests.put")
    @patch("requests.get")
    def test_post_sends_error_message(self, get, put, error):
        get.return_value = Mock(status_code=200)
        put.return_value = Mock(status_code=403, text=u'error')
        data = {
            "old": "old",
            "new": "new",
            "confirm": "new",
        }
        request = RequestFactory().post("/", data)
        request.session = {'tsuru_token': 'tokentest'}
        ChangePassword.as_view()(request)
        error.assert_called_with(request, u'error', fail_silently=True)

    @patch("requests.get")
    def test_login_required(self, get):
        get.return_value = Mock(status_code=200)
        request = RequestFactory().get("/")
        request.session = {}
        response = ChangePassword.as_view()(request)
        self.assertEqual(302, response.status_code)
        self.assertEqual("%s?next=%s" % (reverse('login'), request.path), response.items()[1][1])



from tsuru_dashboard import settings

import requests


class MetricNotEnabled(Exception):
    pass


def get_envs_from_api(app, token):
    headers = {'authorization': token}
    url = "{}/apps/{}/metric/envs".format(settings.TSURU_HOST, app["name"])
    response = requests.get(url, headers=headers)
    if response.status_code == 200:
        return response.json()
    return None



from django.test import TestCase
from django.core.urlresolvers import reverse
from django.conf import settings

from importlib import import_module

from mock import patch, Mock


class NodeRemoveViewTest(TestCase):
    def setUp(self):
        settings.SESSION_ENGINE = 'django.contrib.sessions.backends.file'
        engine = import_module(settings.SESSION_ENGINE)
        store = engine.SessionStore()
        store.save()
        self.session = store
        self.client.cookies[settings.SESSION_COOKIE_NAME] = store.session_key

    @patch("requests.get")
    @patch("requests.delete")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_remove_node_with_default_params(self, token_is_valid, delete, get):
        token_is_valid.return_value = True

        response_mock = Mock(status_code=200)
        delete.return_value = response_mock

        self.session["tsuru_token"] = "admin"
        self.session.save()

        response = self.client.get(
            reverse("node-remove", kwargs={"address": "http://localhost:2345"}))

        self.assertEqual(302, response.status_code)
        self.assertRedirects(response, reverse("pool-list"))

        api_url = "http://localhost:8080/docker/node/http://localhost:2345"
        query = "?remove-iaas=false&no-rebalance=true"
        headers = {'authorization': u'admin'}
        delete.assert_called_with(api_url + query, headers=headers)

    @patch("requests.get")
    @patch("requests.delete")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_remove_node_with_detroy_and_rebalance_true(self, token_is_valid, delete, get):
        token_is_valid.return_value = True

        response_mock = Mock(status_code=200)
        delete.return_value = response_mock

        self.session["tsuru_token"] = "admin"
        self.session.save()

        url = "{}?rebalance=true&destroy=true".format(
            reverse("node-remove", kwargs={"address": "http://localhost:2345"}))
        response = self.client.get(url)

        self.assertEqual(302, response.status_code)
        self.assertRedirects(response, reverse("pool-list"))

        api_url = "http://localhost:8080/docker/node/http://localhost:2345"
        query = "?remove-iaas=true&no-rebalance=false"
        headers = {'authorization': u'admin'}
        delete.assert_called_with(api_url + query, headers=headers)

    @patch("requests.get")
    @patch("requests.delete")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_remove_node_with_detroy_and_rebalance_false(self, token_is_valid, delete, get):
        token_is_valid.return_value = True

        response_mock = Mock(status_code=204)
        delete.return_value = response_mock

        self.session["tsuru_token"] = "admin"
        self.session.save()

        url = "{}?rebalance=false&destroy=false".format(
            reverse("node-remove", kwargs={"address": "http://localhost:2345"}))
        response = self.client.get(url)

        self.assertEqual(302, response.status_code)
        self.assertRedirects(response, reverse("pool-list"))

        api_url = "http://localhost:8080/docker/node/http://localhost:2345"
        query = "?remove-iaas=false&no-rebalance=true"
        headers = {'authorization': u'admin'}
        delete.assert_called_with(api_url + query, headers=headers)

    @patch("requests.delete")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_remove_node_with_error(self, token_is_valid, delete):
        token_is_valid.return_value = True

        response_mock = Mock(status_code=404, text="custom error")
        delete.return_value = response_mock

        self.session["tsuru_token"] = "admin"
        self.session.save()

        url = "{}?rebalance=true&destroy=true".format(
            reverse("node-remove", kwargs={"address": "http://localhost:2345"}))
        response = self.client.get(url)

        self.assertEqual(404, response.status_code)
        self.assertEqual("custom error", response.content)

        api_url = "http://localhost:8080/docker/node/http://localhost:2345"
        query = "?remove-iaas=true&no-rebalance=false"
        headers = {'authorization': u'admin'}
        delete.assert_called_with(api_url + query, headers=headers)

    @patch("requests.get")
    @patch("requests.delete")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_remove_node_with_rebalance_bad_request(self, token_is_valid, delete, get):
        token_is_valid.return_value = True

        response_mock = Mock(status_code=204)
        delete.return_value = response_mock

        self.session["tsuru_token"] = "admin"
        self.session.save()

        url = "{}?rebalance=another&destroy=false".format(
            reverse("node-remove", kwargs={"address": "http://localhost:2345"}))
        response = self.client.get(url)

        self.assertEqual(400, response.status_code)
        self.assertEqual("The value for 'rebalance' parameter should be 'true' or 'false'", response.content)

    @patch("requests.get")
    @patch("requests.delete")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_remove_node_with_destroy_bad_request(self, token_is_valid, delete, get):
        token_is_valid.return_value = True

        response_mock = Mock(status_code=204)
        delete.return_value = response_mock

        self.session["tsuru_token"] = "admin"
        self.session.save()

        url = "{}?rebalance=true&destroy=another".format(
            reverse("node-remove", kwargs={"address": "http://localhost:2345"}))
        response = self.client.get(url)

        self.assertEqual(400, response.status_code)
        self.assertEqual("The value for 'destroy' parameter should be 'true' or 'false'", response.content)



from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

from tsuru_dashboard import settings
from tsuru_dashboard.teams.views import RemoveUser

from mock import patch, Mock


class RemoveUserViewTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().get("/")
        self.request.session = {"tsuru_token": "admin"}

    @patch("django.contrib.messages.error")
    @patch("requests.delete")
    @patch("requests.get")
    def test_view(self, get, delete, error):
        get.return_value = Mock(status_code=200)
        team_name = "avengers"
        user = "username"
        response = RemoveUser.as_view()(self.request, team=team_name,
                                        user=user)
        self.assertEqual(302, response.status_code)
        url = reverse("team-info", args=[team_name])
        self.assertEqual(url, response.items()[1][1])
        url = "{0}/teams/{1}/{2}".format(settings.TSURU_HOST, team_name, user)
        headers = {"authorization": "admin"}
        delete.assert_called_with(url, headers=headers)

    @patch("django.contrib.messages.success")
    @patch("requests.delete")
    @patch("requests.get")
    def test_view_should_send_success_message(self, get, delete, success):
        get.return_value = Mock(status_code=200)
        delete.return_value = Mock(status_code=200)
        team_name = "avengers"
        user = "username"
        RemoveUser.as_view()(self.request, team=team_name,
                             user=user)
        success.assert_called_with(self.request, u'User successfully removed!', fail_silently=True)

    @patch("django.contrib.messages.error")
    @patch("requests.delete")
    @patch("requests.get")
    def test_view_should_send_error_message(self, get, delete, error):
        get.return_value = Mock(status_code=200)
        delete.return_value = Mock(status_code=403, text=u'error')
        team_name = "avengers"
        user = "username"
        RemoveUser.as_view()(self.request, team=team_name,
                             user=user)
        error.assert_called_with(self.request, u'error', fail_silently=True)



from mock import patch
from django.test import TestCase
from django.test.client import RequestFactory
from tsuru_dashboard.components.views import ListComponentJson
import json


class ListComponentJsonViewTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().get("/")
        self.request.session = {"tsuru_token": "token", "permissions": {"admin": True}}

    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_list_components(self, token_is_valid):
        token_is_valid.return_value = True
        expected = {
            "components": [
                "registry",
                "big-sibling"
            ]
        }
        response = ListComponentJson.as_view()(self.request)
        self.assertDictEqual(expected, json.loads(response.content))



from django.template.response import TemplateResponse
from django.shortcuts import redirect
from django.views.generic import TemplateView
from django.core.urlresolvers import reverse
from django.contrib import messages

from tsuru_dashboard import settings
from tsuru_dashboard.auth.views import LoginRequiredView

import requests


class ListService(LoginRequiredView, TemplateView):
    template_name = "services/list.html"

    def get_context_data(self, *args, **kwargs):
        context = super(ListService, self).get_context_data(*args, **kwargs)
        url = "{}/services/instances".format(settings.TSURU_HOST)
        services = []
        resp = requests.get(url, headers=self.authorization)
        if resp.status_code == 200:
            services = resp.json()
        context.update({"services": services})
        return context


class ServiceInstanceDetail(LoginRequiredView, TemplateView):
    template_name = "services/detail.html"

    def apps(self, instance):
        url = "{}/apps".format(settings.TSURU_HOST)
        response = requests.get(url, headers=self.authorization)
        app_list = []
        for app in response.json():
            if app['name'] not in instance['Apps']:
                app_list.append(app['name'])
        return app_list

    def get_instance(self, service_name, instance_name):
        url = "{}/services/{}/instances/{}".format(settings.TSURU_HOST, service_name, instance_name)
        response = requests.get(url, headers=self.authorization)
        return response.json()

    def get_context_data(self, *args, **kwargs):
        context = super(ServiceInstanceDetail, self).get_context_data(*args, **kwargs)
        instance_name = kwargs["instance"]
        service_name = kwargs["service"]
        instance = self.get_instance(service_name, instance_name)
        apps = self.apps(instance)
        context.update(
            {
                "instance": instance,
                "apps": apps,
                "service_name": service_name,
                "instance_name": instance_name
            }
        )

        return context


class ServiceAdd(LoginRequiredView):
    def post(self, request, *args, **kwargs):
        service_name = kwargs["service_name"]
        url = '{}/services/{}/instances'.format(settings.TSURU_HOST, service_name)
        data = {
            "name": request.POST["name"],
            "owner": request.POST["team"],
        }
        response = requests.post(url, data=data, headers=self.authorization)

        if response.status_code == 201:
            messages.success(request, u'Instance was successfully created', fail_silently=True)
            return redirect(reverse('service-list'))

        messages.error(request, response.content, fail_silently=True)
        return redirect(reverse('service-add', args=[service_name]))

    def get(self, request, *args, **kwargs):
        service_name = kwargs["service_name"]
        url = "{}/teams".format(settings.TSURU_HOST)
        response = requests.get(url, headers=self.authorization)
        teams = response.json()
        context = {
            "service": {"name": service_name},
            "teams": teams,
        }
        return TemplateResponse(request, "services/add.html", context)


class Bind(LoginRequiredView):
    def post(self, request, *args, **kwargs):
        app = request.POST["app"]
        instance = kwargs["instance"]
        service = kwargs["service"]

        url = '{}/services/{}/instances/{}/{}'.format(settings.TSURU_HOST, service, instance, app)
        requests.put(url, headers=self.authorization)

        return redirect(reverse('service-detail', args=[service, instance]))


class Unbind(LoginRequiredView):
    def get(self, request, *args, **kwargs):
        app = kwargs["app"]
        instance = kwargs["instance"]
        service = kwargs["service"]

        url = '{}/services/{}/instances/{}/{}'.format(settings.TSURU_HOST, service, instance, app)
        requests.delete(url, headers=self.authorization)

        return redirect(reverse('service-detail', args=[service, instance]))


class ServiceRemove(LoginRequiredView):
    def get(self, request, *args, **kwargs):
        instance = kwargs["instance"]
        service = kwargs["service"]

        url = '{}/services/{}/instances/{}'.format(settings.TSURU_HOST, service, instance)
        requests.delete(url, headers=self.authorization)

        return redirect(reverse('service-list'))
